use crate::{Gain, MAX_CONVERSION_VALUE, REFERENCE_VOLTS};
use physical::quantity::Volts;

/// Raw digital value resulting from the conversion of an analog signal by the ADS1256.
#[cfg_attr(feature = "defmt", derive(defmt::Format))]
#[derive(Copy, Clone, Eq, PartialEq, Ord, PartialOrd)]
#[repr(transparent)]
pub struct Conversion(pub i32);

impl Conversion {
    /// Process the conversion byte reading and return the raw conversion value.
    #[inline]
    pub fn from_reading(reading: [u8; 3]) -> Self {
        let lsb = reading[2] as i32;
        let mb = reading[1] as i32;
        let msb = (reading[0] as i8) as i32; // double cast for sign extension

        Self((msb << 16) | (mb << 8) | lsb)
    }

    pub fn to_voltage(self, gain: Gain) -> Volts<f32> {
        let volts = ((2.0 * REFERENCE_VOLTS) / (MAX_CONVERSION_VALUE as f32))
            * (self.0 as f32 / gain.value() as f32);

        Volts(volts)
    }
}

impl From<i32> for Conversion {
    #[inline(always)]
    fn from(value: i32) -> Self {
        Conversion(value)
    }
}
